/*******************************************************************************

  Defines Types for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    08/15/2004 16:30 - v 5.1
    10/02/2005 16:25 - v 5.2

  Version : 5.2

*******************************************************************************/

#ifndef DIV_TYPE_DEFS_H
#define DIV_TYPE_DEFS_H

#include <windows.h>

//------------------------------------------------------------------------------
// type macros
//------------------------------------------------------------------------------

#define DIM_CHAR char
#define DIM_UCHAR unsigned char
#define DIM_INT int
#define DIM_INT16 short
#define DIM_INT32 int

#ifdef WIN32
#define DIM_INT64 __int64
#else
#define DIM_INT64 long long
#endif

#define DIM_UINT unsigned int
#define DIM_UINT16 unsigned short
#define DIM_UINT32 unsigned long
#ifdef WIN32
#define DIM_UINT64 unsigned __int64
#else
#define DIM_UINT64 unsigned long long
#endif

#define DIM_LONG long
#define DIM_ULONG unsigned long
#define DIM_DOUBLE double
#define DIM_UDOUBLE unsigned double
#define DIM_BOOL bool
#define DIM_FLOAT float

//------------------------------------------------------------------------------
// HERE DEFINED DATA TYPES IN CONFORMANCE WITH TIFF STRUCTURE
// This types are used in tiff tags and many independent structures
// like EXIF, let's stick to it as well
//------------------------------------------------------------------------------

typedef enum {
  DIM_TAG_POINTER   = 0,  // 32-bit pointer
  DIM_TAG_BYTE      = 1,  // 8-bit unsigned integer
  DIM_TAG_ASCII     = 2,  // 8-bit bytes w/ last byte null
  DIM_TAG_SHORT     = 3,  // 16-bit unsigned integer
  DIM_TAG_LONG      = 4,  // 32-bit unsigned integer
  DIM_TAG_RATIONAL  = 5,  // 64-bit unsigned fraction
  DIM_TAG_SBYTE     = 6,  // 8-bit signed integer
  DIM_TAG_UNDEFINED = 7,  // 8-bit untyped data
  DIM_TAG_SSHORT    = 8,  // 16-bit signed integer
  DIM_TAG_SLONG     = 9,  // 32-bit signed integer
  DIM_TAG_SRATIONAL = 10, // 64-bit signed fraction
  DIM_TAG_FLOAT     = 11, // 32-bit IEEE floating point
  DIM_TAG_DOUBLE    = 12, // 64-bit IEEE floating point
  DIM_TAG_BOOL      = 13, // 8-bit boolean value  
  DIM_TAG_ENUM      = 14  // 32-bit unsigned integer (creates ComboBox in filter params)
} TDIVTagType;

const int divTypeSizeBytes[15] = {4, 1, 1, 2, 4, 8, 1, 1, 2, 4, 8, 4, 8, 1, 4};


#endif //DIV_TYPE_DEFS_H
