/*****************************************************************************
  Interface for Dave Coffin's DCRAW (modified)
  
  Copyright (c) 2005 by Dmitry V. Fedorov <www.dimin.net> <dima@dimin.net>

  IMPLEMENTATION
  
  Programmer: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>

  TODO:
    CR2 - thumbnail
    ORF - thumbnail
    X3F - thumbnail


  History:
    05/30/2005 14:48 - First creation
    05/30/2005 22:43 - update for dcraw 1.262 + metadata and thumbnails
                       X3F - metadata-text
    07/21/2006 19:41 - updated to 1.334 (8.25)                       
                         
  Ver : 4
*****************************************************************************/

#include <xstring.h>


#ifndef NO_JPEG
  #pragma message( ">>> libjpeg inclusion" )

  #include <stdio.h> // jpeglib needs this to be pre-included
  #include <setjmp.h>

  #ifdef FAR
  #undef FAR
  #endif

  //#if defined(__RPCNDR_H__)
  #define HAVE_BOOLEAN
  #define boolean unsigned int
  //#endif

  extern "C" {
    #define XMD_H // Shut JPEGlib up.

    #include <jpeglib.h>
    #include <jerror.h>

    #undef boolean

    #ifdef const
    #undef const // Remove crazy C hackery in jconfig.h
    #endif
  }

#endif


#include <wchar.h>
#include <direct.h>
#include <stdio.h>
#include <io.h>


#include <string>

std::string metatxt;




typedef unsigned char *DimGetHBScanLine( unsigned int y, unsigned char *p, unsigned int w, unsigned int depth );
typedef int (__stdcall *DimTestAbortProc)( void );

void (__stdcall *progress_fun)(long done, long total, char *descr) = NULL;
DimTestAbortProc dim_test_abort = NULL;

void do_progress (long done, long total, char *descr);
bool test_abort ( );


//----------------------------------------------------------------------------
// stubs for typcasting in dcraw
//----------------------------------------------------------------------------
extern "C" {
int get_jpeg_thumb_info( unsigned int *w, unsigned int *h, FILE *ifp )
{
  *w = 0;
  *h = 0;

  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;
  cinfo.err = jpeg_std_error (&jerr);

  jpeg_CreateDecompress ( &cinfo, JPEG_LIB_VERSION, sizeof(cinfo) );

  jpeg_stdio_src (&cinfo, ifp);
  jpeg_read_header (&cinfo, TRUE);

  *w = cinfo.output_width;
  *h = cinfo.output_height;

  return 0;
}
} // extern "C" {


//----------------------------------------------------------------------------
// stubs for typcasting in dcraw
//----------------------------------------------------------------------------
#define strcmp(s1, s2) strcmp((const char *)s1, (const char *) s2)
#define swab(s, d, n) swab((char *) s, (char *) d, n)
//#define foveon_avg(p, r, c) foveon_avg((short *) p, r, c)

#ifndef M_PI
#define M_PI        3.14159265358979323846
#endif

#define NO_LCMS
#define DJGPP

//----------------------------------------------------------------------------
// include DCRAW code
//----------------------------------------------------------------------------
#define main main_dcraw
#include "dcraw.c"
#undef main

#undef boolean


//----------------------------------------------------------------------------
// UI supprot functions
//----------------------------------------------------------------------------

void do_progress (long done, long total, char *descr) {
  if (progress_fun != NULL) progress_fun( done, total, descr);
}

bool test_abort ( ) {
  if (dim_test_abort != NULL) 
    if (dim_test_abort( ) == 1) return true;
  return false;
}

//----------------------------------------------------------------------------
// misc
//----------------------------------------------------------------------------

inline void set_fd( FILE *new_fd ) {
  ifp = new_fd;
}

FILE *init_read( char *fname ) {
  if (!(ifp = fopen (fname, "rb"))) return NULL;
  return ifp;
}

inline unsigned char trimToUCv( int v ) {
  unsigned char r = v;
  if (v>255) r = 255;
  return r;
}

int get_info( unsigned int *w, unsigned int *h, xstring &make_str ) {

  fseek (ifp, 0, SEEK_SET);
  identify();

  shrink = (half_size || threshold) && filters;
  iheight = (height + shrink) >> shrink;
  iwidth  = (width  + shrink) >> shrink;

  if (is_raw) {
    //*h = height;
    //*w = width;
    *h = iheight;
    *w = iwidth;
    make_str = make;
  }

  return is_raw;
}

int get_info_thumb( unsigned int *w, unsigned int *h ) {

  fseek (ifp, 0, SEEK_SET);  
  identify();
  
  if (thumb_offset) {
    *h = thumb_height;
    *w = thumb_width;
  }

  return thumb_offset;
}


//----------------------------------------------------------------------------
// DCRAW reader for windows BITMAP
//----------------------------------------------------------------------------

int read_cdraw_image( unsigned char **pbuf, DimGetHBScanLine scanLine, void *prog_proc, int quality, DimTestAbortProc abortProc )
{
  write_fun = write_ppm_tiff;

  //-----------------------------------------------
  // config
  //-----------------------------------------------
  int half_size=0;
  int user_flip=-1;
  //int quality=1;

  use_auto_wb = 0;
  use_camera_wb = 1;
  output_color  = 1;
  no_auto_bright = 1;
  //-----------------------------------------------


  if (ifp == NULL) return 1;
  progress_fun = (void (__stdcall *)(long,long,char *)) prog_proc;
  dim_test_abort = abortProc;

  if (is_foveon) no_auto_bright = 0;

  shrink = (half_size || threshold) && filters;
  iheight = (height + shrink) >> shrink;
  iwidth  = (width  + shrink) >> shrink;
	//iheight *= ymag;
	//iwidth  *= xmag;

  int use_camera_matrix=0;
  if (use_camera_matrix && cmatrix[0][0] > 0.25) {
    memcpy (rgb_cam, cmatrix, sizeof cmatrix);
    raw_color = 0;
  }

  //image = (unsigned short (*)[4]) calloc (iheight * iwidth * sizeof(*image) + meta_length, 1);
  image = (unsigned short (*)[4]) malloc (iheight * iwidth * sizeof(*image) + meta_length);
  if (image == NULL) return 1;
  meta_data = (char *) (image + iheight*iwidth);

  if (verbose)
    fprintf (stderr, "Loading %s %s image from %s...\n", make, model, ifname);

  fseek (ifp, data_offset, SEEK_SET);
  (*load_raw)();
  if ( test_abort() ) return 1;
  
  if (zero_is_bad) remove_zeroes();
  //if (dark_frame) subtract (dark_frame);

  height = iheight;
  width  = iwidth;

  #ifdef COLORCHECK
  colorcheck();
  #endif

  if ( test_abort() ) return 1;
  
  if (is_foveon && !document_mode) foveon_interpolate();
  if (!is_foveon && document_mode < 2) scale_colors();
  pre_interpolate();
  //if (shrink) filters = 0;
  //cam_to_cielab (NULL,NULL);

  if ( test_abort() ) return 1;
  if (filters && !document_mode) {
    if (quality == 0)	lin_interpolate();
    else if (quality == 1 || colors > 3) vng_interpolate();
    else if (quality == 2) ppg_interpolate();
    else ahd_interpolate();
  }

  int i;
  if (mix_green)
    for (colors=3, i=0; i < height*width; i++)
	    image[i][1] = (image[i][1] + image[i][3]) >> 1;

  if ( test_abort() ) return 1;
  //if (sigma_d > 0 && sigma_r > 0) bilateral_filter();
  if (!is_foveon && colors == 3) median_filter();
  if (!is_foveon && highlight == 2) blend_highlights();
  if (!is_foveon && highlight > 2) recover_highlights();
  //if (use_fuji_rotate) fuji_rotate();
  #ifndef NO_LCMS
  if (cam_profile) apply_profile (cam_profile, out_profile);
  #endif

  if ( test_abort() ) return 1;
  convert_to_rgb();
  //if (flip) flip_image();

  if ( test_abort() ) return 1;

  //------------------------------------------------------------------------------------
  // code from write_ppm
  //------------------------------------------------------------------------------------  
  uchar lut[0x10000];
  gamma_lut (lut);

  // create actual BITMAP
  unsigned char *p = *pbuf;
  unsigned long x, y;
  unsigned long ipos=0;
/*
  for (y=0; y<height; y++) 
  {
    do_progress (y, height, "DCRAW: Scaling 16 bit");
    if ( test_abort() ) return 1;
    p = scanLine( height-y-1, *pbuf, width, 24 );
    
    ipos = y*width;
    for (x=0; x<width; x++) 
    {
      p[2] = trimToUCv( lut[image[ipos][0]] );
      p[1] = trimToUCv( lut[image[ipos][1]] );
      p[0] = trimToUCv( lut[image[ipos][2]] );
      p+=3;
      ipos++;
    }
  }
*/

  int soff  = flip_index (0, 0);
  int cstep = flip_index (0, 1) - soff;
  int rstep = flip_index (1, 0) - flip_index (0, width);
  for (int row=0; row < height; row++, soff += rstep) {
    
    do_progress (row, height, "DCRAW: Scaling 16 bit");
    if ( test_abort() ) return 1;
    p = scanLine( height-row-1, *pbuf, width, 24 );    

    for (int col=0; col < width; col++, soff += cstep) {
      p[2] = trimToUCv( lut[image[soff][0]] );
      p[1] = trimToUCv( lut[image[soff][1]] );
      p[0] = trimToUCv( lut[image[soff][2]] );
      p+=3;
    }
  }


  free(image);
  image = NULL;
  return 0;
}



//****************************************************************************
// include PARSE code
//****************************************************************************


//int thumb_offset, thumb_length, 
int thumb_layers;
char thumb_head[128];
int offset, length, bps;
int is_dng;

#define parse_tiff_ifd   parse_tiff_ifd_thumbs
#define parse_tiff       parse_tiff_thumbs
#define parse_minolta    parse_minolta_thumbs
#define parse_mos        parse_mos_thumbs
#define parse_rollei     parse_rollei_thumbs
#define parse_foveon     parse_foveon_thumbs
#define parse_fuji       parse_fuji_thumbs
#define parse_phase_one  parse_phase_one_thumbs
#define parse_jpeg       parse_jpeg_thumbs

#define main             main_parse
#define identify         identify_thumbs
#define parse_makernote  parse_makernote_parse
#define parse_exif       parse_exif_parse
#define sony_decrypt     sony_decrypt_parse

#define sget2(s) sget2((uchar *)s)
#define sget4(s) sget4((uchar *)s)
#define strncmp(s1, s2, c) strcmp((const char *)s1, (const char *) s2, c)

#include "parse.c"

#undef main
#undef identify


char *parse_foveon_metatext()
{
  int entries, img=0, off, len, tag, save, i, j, k, pent, poff[256][2];
  char name[128], value[128], camf[0x20000], *pos, *cp, *dp;
  unsigned val, key, type, num, ndim, dim[3];

  //std::string metatxt;

  metatxt = "\n\n[SIGMA METADATA]\n";
  
  order = 0x4949;     /* Little-endian */
  fseek (ifp, -4, SEEK_END);
  fseek (ifp, get4(), SEEK_SET);
  if (get4() != 0x64434553) { /* SECd */
    printf ("Bad Section identifier at %6x\n", (int)ftell(ifp)-4);
    return NULL;
  }
  get4();
  entries = get4();
  while (entries--) 
  {
    off = get4();
    len = get4();
    tag = get4();
    save = ftell(ifp);
    fseek (ifp, off, SEEK_SET);
    //printf ("%c%c%c%c at offset %06x, length %06x, ", tag, tag >> 8, tag >> 16, tag >> 24, off, len);
    if (get4() != (0x20434553 | (tag << 24))) {
      printf ("Bad Section identifier at %6x\n", off);
      goto next;
  }
    
  val = get4();
  //printf ("version %d.%d, ",val >> 16, val & 0xffff);
  switch (tag) {
  case 0x32414d49:      /* IMA2 */
  case 0x47414d49:      /* IMAG */
    if (++img == 2) {   /* second image */
      thumb_offset = off;
      thumb_length = 1;
    }
    //printf ("type %d, " , get4());
    //printf ("format %2d, "  , get4());
    //printf ("columns %4d, " , get4());
    //printf ("rows %4d, "  , get4());
    //printf ("rowsize %d\n"  , get4());
    break;
  case 0x464d4143:      /* CAMF */
    //printf ("type %d, ", get4());
    get4();
    for (i=0; i < 4; i++)
      putchar(fgetc(ifp));
    val = get4();
    //printf (" version %d.%d:\n",val >> 16, val & 0xffff);
    key = get4();
    if ((len -= 28) > 0x20000)
      len = 0x20000;
    fread (camf, 1, len, ifp);
    for (i=0; i < len; i++) {
      key = (key * 1597 + 51749) % 244944;
      val = key * (INT64) 301593171 >> 24;
      camf[i] ^= ((((key << 8) - val) >> 1) + val) >> 17;
    }
    for (pos=camf; (unsigned) (pos-camf) < len; pos += sget4(pos+8)) {
      if (strncmp (pos, "CMb", 3)) {
        printf("Bad CAMF tag \"%.4s\"\n", pos);
        break;
      }
      val = sget4(pos+4);
      printf ("  %4.4s version %d.%d: ", pos, val >> 16, val & 0xffff);
      switch (pos[3]) {
      case 'M':
        cp = pos + sget4(pos+16);
        type = sget4(cp);
        ndim = sget4(cp+4);
        dim[0] = dim[1] = dim[2] = 1;
        printf ("%d-dimensonal array %s of type %d:\n    Key: (",
          ndim, pos+sget4(pos+12), sget4(cp));
        dp = pos + sget4(cp+8);
        for (i=ndim; i--; ) {
          cp += 12;
          dim[i] = sget4(cp);
          printf ("%s %d%s", pos+sget4(cp+4), dim[i], i ? ", ":")\n");
        }
        for (i=0; i < dim[2]; i++) {
          for (j=0; j < dim[1]; j++) {
            printf ("    ");
            for (k=0; k < dim[0]; k++)
              switch (type) {
      case 0:
      case 6:
        printf ("%7d", sget2(dp));
        dp += 2;
        break;
      case 1:
      case 2:
        printf (" %d", sget4(dp));
        dp += 4;
        break;
      case 3:
        val = sget4(dp);
        printf (" %9f", *(float *)(&val));
        dp += 4;
            }
            printf ("\n");
          }
          printf ("\n");
        }
        break;
      case 'P':
        val = sget4(pos+16);
        num = sget4(pos+val);
        printf ("%s, %d parameters:\n", pos+sget4(pos+12), num);
        cp = pos+val+8 + num*8;
        for (i=0; i < num; i++) {
          val += 8;
          printf ("    %s = %s\n", cp+sget4(pos+val), cp+sget4(pos+val+4));
        }
        break;
      case 'T':
        cp = pos + sget4(pos+16);
        printf ("%s = %.*s\n", pos+sget4(pos+12), sget4(cp), cp+4);
        break;
      default:
        printf ("\n");
      }
    }
    break;
  case 0x504f5250:      /* PROP */
    printf ("entries %d, ", pent=get4());
    printf ("charset %d, ", get4());
    get4();
    printf ("nchars %d\n", get4());
    off += pent*8 + 24;
    if (pent > 256) pent=256;
    for (i=0; i < pent*2; i++)
      poff[0][i] = off + get4()*2;
    for (i=0; i < pent; i++) {
      get_utf8 (poff[i][0], name, 128);
      get_utf8 (poff[i][1], value, 128);
      printf ("  %s = %s\n", name, value);
      
      metatxt += "  ";
      metatxt += name;
      metatxt += " = ";
      metatxt += value;
      metatxt += "\n";

      if (!strcmp (name,"CAMMANUF"))
        strcpy (make, value);
      if (!strcmp (name,"CAMMODEL"))
        strcpy (model, value);
    }
  }

next:
  fseek (ifp, save, SEEK_SET);
}

  return (char *) metatxt.c_str();
}


char *read_dcraw_metatext( )
{
  xstring str;
  int c, i;

  if (make[0]) {
    str.saddf("Camera: %s %s\n", make, model);
    str.saddf("Timestamp: %s", ctime(&timestamp));
    str.saddf("ISO speed: %d\n", (int) iso_speed);
    str += "Exposure: ";
    if (shutter > 0 && shutter < 1) {
      str += "1/";
	    shutter = 1.0 / shutter;
    }
    str.saddf("%0.1f sec\n", shutter);
    str.saddf("Aperture: F%0.1f\n", aperture);
    str.saddf("Focal length: %d mm\n", (int) focal_len);
    //str.saddf("Secondary pixels: %s\n", fuji_secondary ? "yes":"no");
    str.saddf("Embedded ICC profile: %s\n", profile_length ? "yes":"no");

    str.saddf("Full size:   %4d x %d\n", raw_width, raw_height);
    if (thumb_offset)
	    str.saddf ("Thumb size:  %4d x %d\n", thumb_width, thumb_height);

    str.saddf("Raw colors: %d", colors);
    str += "\nDaylight multipliers:";
    FORCC str.saddf(" %0.4f", pre_mul[c]);

	  if (cam_mul[0] > 0) {
	    str += ("\nCamera multipliers:");
	    FORC4 str.saddf (" %0.2f", cam_mul[c]);
    }

	  if (filters) {
	    str += ("\nFilter pattern: ");
	    if (!cdesc[3]) cdesc[3] = 'G';
	    for (i=0; i < 32; i+=2)
	      str.saddf ( "%c", cdesc[filters >> i & 3]);
	  }
  }


  // individual metadata
  if (strncmp(make, "SIGMA", 5) == 0) {
    str += parse_foveon_metatext();
  }



  // prepare the buffer
  char *cstr = new char [str.size()+1];
  cstr[str.size()] = 0;
  memcpy( cstr, str.c_str(), str.size() );
  return cstr;
}


//----------------------------------------------------------------------------
// THUMBNAIL
//----------------------------------------------------------------------------

int foveon_decode_thumbnail ( unsigned char **pbuf, DimGetHBScanLine scanLine )
{
  unsigned char *p = *pbuf;
  
  int bwide, row, col, bit=-1, c, i;
  char *buf;
  struct decode *dindex;
  short pred[3];
  unsigned bitbuf=0;
 
  fseek (ifp, thumb_offset, SEEK_SET);
  bwide  = get4();
  if (bwide > 0) {
    buf = (char *) malloc(bwide);
    for (row=0; row < height; row++) {
      fread  (buf, 1, bwide, ifp);
      //fwrite (buf, 3, width, tfp);
    }
    free (buf);
    return 1;
  }
  
  foveon_decoder (256, 0);

  for (row=0; row < thumb_height; row++) {
    p = scanLine( thumb_height-row-1, *pbuf, width, 24 );
    memset (pred, 0, sizeof pred);
    if (!bit) get4();
    for (col=bit=0; col < thumb_width; col++) {
      for (c=0; c < 3; c++) {
        for (dindex=first_decode; dindex->branch[0]; ) {
         if ((bit = (bit-1) & 31) == 31)
         for (i=0; i < 4; i++)
           bitbuf = (bitbuf << 8) + fgetc(ifp);
         dindex = dindex->branch[bitbuf >> bit & 1];
        }
        pred[c] += dindex->leaf;

        p[2-c] = pred[c];
        //fputc (pred[c], tfp);
      }
      p+=3;
    }
  }

  return 0;
}


int kodak_decode_thumbnail ( unsigned char **pbuf, DimGetHBScanLine scanLine )
{
  uchar c, blen[384];
  unsigned row, col, len, bits=0;
  INT64 bitbuf=0;
  int i, li=0, si, diff, six[6], y[4], cb=0, cr=0, rgb[3];
  ushort *out, *op;
  
  fseek (ifp, thumb_offset, SEEK_SET);
  width = (width+1) & -2;
  height = (height+1) & -2;
  //fprintf (tfp, "P6\n%d %d\n65535\n", width, height);
  out = (unsigned short *) malloc (width * 12);
  if (!out) {
    fprintf (stderr, "kodak_yuv_decode() malloc failed!\n");
    return 1;
  }
  
  for (row=0; row < height; row+=2) 
  {
    for (col=0; col < width; col+=2) 
    {
      if ((col & 127) == 0) 
      {
        len = (width - col + 1) * 3 & -4;
        if (len > 384) len = 384;
        for (i=0; i < len; ) 
        {
          c = fgetc(ifp);
          blen[i++] = c & 15;
          blen[i++] = c >> 4;
        }
        li = bitbuf = bits = y[1] = y[3] = cb = cr = 0;
        if (len % 8 == 4) 
        {
          bitbuf  = fgetc(ifp) << 8;
          bitbuf += fgetc(ifp);
          bits = 16;
        }
      }
      for (si=0; si < 6; si++) 
      {
        len = blen[li++];
        if (bits < len) 
        {
          for (i=0; i < 32; i+=8)
            bitbuf += (INT64) fgetc(ifp) << (bits+(i^8));
          bits += 32;
        }
        diff = bitbuf & (0xffff >> (16-len));
        bitbuf >>= len;
        bits -= len;
        if ((diff & (1 << (len-1))) == 0)
          diff -= (1 << len) - 1;
        six[si] = diff;
      }
      y[0] = six[0] + y[1];
      y[1] = six[1] + y[0];
      y[2] = six[2] + y[3];
      y[3] = six[3] + y[2];
      cb  += six[4];
      cr  += six[5];
      for (i=0; i < 4; i++) 
      {
        op = out + ((i >> 1)*width + col+(i & 1)) * 3;
        rgb[0] = y[i] + 1.40200/2 * cr;
        rgb[1] = y[i] - 0.34414/2 * cb - 0.71414/2 * cr;
        rgb[2] = y[i] + 1.77200/2 * cb;
        for (c=0; c < 3; c++)
          if (rgb[c] > 0) op[c] = htons(rgb[c]);
      }
    }
    //fwrite (out, sizeof *out, width*6, tfp);
  }
  free(out);
  
  return 0;
}

int rollei_decode_thumbnail (unsigned char **pbuf, DimGetHBScanLine scanLine)
{
  int row, col;
  int i, size = thumb_width * thumb_height;
  ushort *thumb = (ushort *) malloc (size*2);
  
  unsigned char *p = *pbuf;
  fseek (ifp, thumb_offset, SEEK_SET);

  i = 0;
  read_shorts (thumb, size);
  for (row=0; row < thumb_height; row++)
    for (col=0; col < thumb_width; col++) {
      p = scanLine( height-row-1, *pbuf, width, 24 );
      if (i >= size) break;      
      p[2] = thumb[i] << 3;
      p[1] = thumb[i] >> 5  << 2;
      p[0] = thumb[i] >> 11 << 3;
      ++i;
    }
  free (thumb);
  return 0;
}


//----------------------------------------------------------------------------
// libJPEG memory manager 
//----------------------------------------------------------------------------

static const int max_buf = 4096;

struct mem_jpeg_source_mgr : public jpeg_source_mgr {
    unsigned char *img;
    unsigned int  pos, size; // in bytes
    JOCTET buffer[max_buf];
public:
    mem_jpeg_source_mgr(unsigned char *_img, unsigned int _size);
};

extern "C" {

static void mem_jpeg_init_source(j_decompress_ptr) {
}

static unsigned int mem_jpeg_fill_input_buffer(j_decompress_ptr cinfo) {
  unsigned int num_read;
  mem_jpeg_source_mgr* src = (mem_jpeg_source_mgr*)cinfo->src;
  src->next_input_byte = src->buffer;
  
  num_read = (max_buf < src->size-src->pos) ? max_buf : src->size-src->pos;

  if (num_read > 0) {
    memcpy( src->buffer, src->img+src->pos, num_read );
    src->pos += num_read;
    src->bytes_in_buffer = num_read;
  } else {
    // Insert a fake EOI marker - as per jpeglib recommendation
    src->buffer[0] = (JOCTET) 0xFF;
    src->buffer[1] = (JOCTET) JPEG_EOI;
    src->bytes_in_buffer = 2;
  } 

  return true;
}

static void mem_jpeg_skip_input_data(j_decompress_ptr cinfo, long num_bytes) {
  mem_jpeg_source_mgr* src = (mem_jpeg_source_mgr*) cinfo->src;

  if (num_bytes > 0) {
    while (num_bytes > (long) src->bytes_in_buffer) {
      num_bytes -= (long) src->bytes_in_buffer;
      (void) mem_jpeg_fill_input_buffer(cinfo);
    }
    src->next_input_byte += (size_t) num_bytes;
    src->bytes_in_buffer -= (size_t) num_bytes;
  }
}

static void mem_jpeg_term_source(j_decompress_ptr) {
}

} // extern C


inline mem_jpeg_source_mgr::mem_jpeg_source_mgr(unsigned char *_img, unsigned int _size) {
  jpeg_source_mgr::init_source       = mem_jpeg_init_source;
  jpeg_source_mgr::fill_input_buffer = mem_jpeg_fill_input_buffer;
  jpeg_source_mgr::skip_input_data   = mem_jpeg_skip_input_data;
  jpeg_source_mgr::resync_to_restart = jpeg_resync_to_restart;
  jpeg_source_mgr::term_source       = mem_jpeg_term_source;
  img             = _img;
  size            = _size;
  pos             = 0;
  bytes_in_buffer = 0;
  next_input_byte = buffer;
}

//-----------------------------------------------------------------------------------

int jpeg_decode_thumbnail ( unsigned char **pbuf, DimGetHBScanLine scanLine )
{
  // read whole thumb to memory
  unsigned char *thumb = (unsigned char *) malloc (thumb_length);
  fseek (ifp, thumb_offset, SEEK_SET);
  fread (thumb, 1, thumb_length, ifp);
  thumb[0] = 0xff;

  if (memcmp(thumb,"\377\330\377",3) != 0) return 1;


  // decode thumb in memory
  JSAMPROW row_pointer[1];

  struct jpeg_decompress_struct cinfo;
  struct mem_jpeg_source_mgr *iod_src = new mem_jpeg_source_mgr( thumb, thumb_length );
  struct jpeg_error_mgr jerr;

  jpeg_create_decompress(&cinfo);
  cinfo.src = iod_src;
  cinfo.err = jpeg_std_error(&jerr);
  //jerr.error_exit = my_error_exit;

  //if (!setjmp(jerr.setjmp_buffer)) {

    (void) jpeg_read_header(&cinfo, TRUE);
    (void) jpeg_start_decompress(&cinfo);

    if ( cinfo.output_components == 3 ) {
      row_pointer[0] = new uchar[cinfo.output_width*cinfo.output_components];

      while (cinfo.output_scanline < cinfo.output_height) {
        register unsigned int i;        
      
        (void) jpeg_read_scanlines( &cinfo, row_pointer, 1 );
        uchar *row = row_pointer[0];
        uchar *pix = scanLine( cinfo.output_height-cinfo.output_scanline, *pbuf, cinfo.output_width, 24 );

        // windows bitmap consists of BGR tripls, so need to invert
        for (i=0; i<cinfo.output_width; i++) {
          pix[2] = *row++;
          pix[1] = *row++;
          pix[0] = *row++;
          pix += 3;
        }

      } // while scanlines
      delete [] row_pointer[0];
    } // if 3 components
    
    (void) jpeg_finish_decompress(&cinfo);

  //} // if (!setjmp(jerr.setjmp_buffer))


  jpeg_destroy_decompress(&cinfo);
  free (thumb);
  delete iod_src;

  return 0;
}

int read_dcraw_thumb( unsigned char **pbuf, DimGetHBScanLine scanLine )
{
  if (ifp == NULL) return 1;
  progress_fun   = NULL;
  dim_test_abort = NULL;

/*  
  if (thumb_load_raw) {
	  load_raw = thumb_load_raw;
	  data_offset = thumb_offset;
	  height = thumb_height;
	  width  = thumb_width;
    image = (unsigned short (*)[4]) malloc (height * width * sizeof(*image) );
    if (image == NULL) return 1;
    fseek (ifp, data_offset, SEEK_SET);
    (*load_raw)();

  (*write_fun)(ofp);

  //------------------------------------------------------------------------------------
  // code from write_ppm
  //------------------------------------------------------------------------------------  
  uchar lut[0x10000];
  gamma_lut (lut);


  // create actual BITMAP
  unsigned char *p = *pbuf;
  unsigned long x, y;
  unsigned long ipos=0;

  for (y=0; y<height; y++) 
  {
    do_progress (y, height, "DCRAW: Scaling 16 bit");
    if ( test_abort() ) return 1;
    p = scanLine( height-y-1, *pbuf, width, 24 );
    
    ipos = y*width;
    for (x=0; x<width; x++) 
    {
      p[2] = trimToUCv( lut[image[ipos][0]] );
      p[1] = trimToUCv( lut[image[ipos][1]] );
      p[0] = trimToUCv( lut[image[ipos][2]] );
      p+=3;
      ipos++;
    }
  }

  free(image);
  image = NULL;


  } else {
*/

    height = thumb_height;
    width = thumb_width;
	  fseek (ifp, thumb_offset, SEEK_SET);
    
    if (write_thumb == jpeg_thumb) 
      return jpeg_decode_thumbnail ( pbuf, scanLine );

    if (write_thumb == foveon_thumb) 
      return foveon_decode_thumbnail ( pbuf, scanLine );

    if (write_thumb == rollei_thumb) 
      return rollei_decode_thumbnail ( pbuf, scanLine );

    return 1;

//layer_thumb
//ppm_thumb


  //}

}












