/*****************************************************************************
  Digital Cameras RAW file support
  Copyright (c) 2004 by Dmitry V. Fedorov <www.dimin.net> <dima@dimin.net>

  IMPLEMENTATION
  
  Programmer: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>

  Note:
    Now supports Big-Endian and Small-Endian arcitectures

  History:
    10/26/2004 19:00 - First creation
        
  Ver : 1
*****************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "dcraw.cpp"

#include "dim_dcraw_format.h"

const int pos_to_quality[4] = { 0, 25, 50, 100 };

//****************************************************************************
// MISC
//****************************************************************************

TDimDCRawParams initTDimDCRawParams()
{
  TDimDCRawParams r;

  r.i = initTDivImageInfo();
  r.i.depth  = 8;

  return r;
}

//****************************************************************************
//
// INTERNAL STRUCTURES
//
//****************************************************************************

void dimDCRawGetImageInfo( TDivFormatHandle *fmtHndl )
{


}

//****************************************************************************
//
// FORMAT DEMANDED FUNTIONS
//
//****************************************************************************


//----------------------------------------------------------------------------
// PARAMETERS, INITS
//----------------------------------------------------------------------------

unsigned char canon_magic1[4] = { 0x49, 0x49, 0x1a, 0x00 };
unsigned char canon_magic2[4] = { 0x4d, 0x4d, 0x1a, 0x00 };

DIM_INT __stdcall dimDCRawValidateFormatProc (void *magic, DIM_UINT32 length)
{
  if (length < 10) return -1;
  unsigned char *mag_num = (unsigned char *) magic;

  //II* - TIFF
  //II* - NEF Nikon
  //II* - DNG
  if (memcmp (mag_num, canon_magic1, 4) == 0 ) return DCRAW_FORMAT_CANON; // II 
  if (memcmp (mag_num, canon_magic2, 4) == 0 ) return DCRAW_FORMAT_CANON; 
  if (memcmp (mag_num, "MMMMRawT", 8)   == 0 ) return DCRAW_FORMAT_DCRAW;
  if (memcmp (mag_num, "IIII", 4)       == 0 ) return DCRAW_FORMAT_DCRAW;
  if (memcmp (mag_num, "\0MRM", 4)      == 0 ) return DCRAW_FORMAT_MINOLTA; // mrw - Konica/Minolta
  if (memcmp (mag_num, "IIRS", 4)       == 0 ) return DCRAW_FORMAT_OLYMPUS; // orf - Olympus
  if (memcmp (mag_num, "PXN", 3)        == 0 ) return DCRAW_FORMAT_DCRAW; // Logitech
  if (memcmp (mag_num, "FUJIFILM", 8)   == 0 ) return DCRAW_FORMAT_FUJI;
  if (memcmp (mag_num, "DSC-Image", 9)  == 0 ) return DCRAW_FORMAT_ROLLEI; // rollei
  if (memcmp (mag_num, "FOVb", 4)       == 0 ) return DCRAW_FORMAT_SIGMA; // x3f - Sigma / foveon chip
  //if (memcmp (mag_num, "\xff\xd8\xff\xe1", 4)       == 0 ) return 0; // PENTAX jpeg - let's use original jpeg reader

  if (length >= 14) 
    if (memcmp (mag_num+6, "HEAPCCDR", 8) == 0  ) return DCRAW_FORMAT_DCRAW;

  if (length >= 32) 
    if (memcmp (mag_num+25, "ARECOYK", 7) == 0  ) return DCRAW_FORMAT_DCRAW; //Contax


  //if (memcmp (mag_num, "BM", 2) ) return 0; // BMQ got same magic as bmp...
  //if (memcmp (mag_num, "BR", 2) ) return 0;

  return -1;
}

TDivFormatHandle __stdcall dimDCRawAquireFormatProc( void )
{
  TDivFormatHandle fp = initTDivFormatHandle();
  fp.quality = 0;
  dimDCRawLoadRegProc ( &fp );
  return fp;
}

void __stdcall dimDCRawReleaseFormatProc (TDivFormatHandle *fmtHndl)
{
  if (fmtHndl == NULL) return;
  //dimDCRawSaveRegProc ( fmtHndl );
  dimDCRawCloseImageProc ( fmtHndl );  
}


//----------------------------------------------------------------------------
// OPEN/CLOSE
//----------------------------------------------------------------------------
void __stdcall dimDCRawCloseImageProc (TDivFormatHandle *fmtHndl)
{
  if (fmtHndl == NULL) return;
  dimClose ( fmtHndl );
  if (fmtHndl->internalParams != NULL) delete fmtHndl->internalParams;
  fmtHndl->internalParams = NULL;
}

DIM_UINT __stdcall dimDCRawOpenImageProc  (TDivFormatHandle *fmtHndl, TDIVIOModes io_mode)
{
  if (fmtHndl == NULL) return 1;
  if (fmtHndl->internalParams != NULL) dimDCRawCloseImageProc (fmtHndl);  
  fmtHndl->internalParams = (void *) new TDimDCRawParams [1];
  TDimDCRawParams *picPar = (TDimDCRawParams *) fmtHndl->internalParams;
  *picPar = initTDimDCRawParams();

  if (io_mode == DIV_IO_READ)
  {
    image = NULL;    
    fmtHndl->stream = fopen( fmtHndl->fileName, "rb" );
    if (fmtHndl->stream == NULL) return 1;
    
    set_fd( (FILE *) fmtHndl->stream );
    
    unsigned int w, h;
    xstring make_string;
    if ( get_info( &w, &h, make_string ) == 1 )
    {
      picPar->i.width   = w;
      picPar->i.height  = h;
      picPar->i.depth   = 8;
      picPar->i.samples = 3;
      /*
      make_string = make_string.toLowerCase();
      fmtHndl->subFormat = DCRAW_FORMAT_DCRAW;
      if (make_string == "canon") fmtHndl->subFormat = DCRAW_FORMAT_CANON;
      if (make_string == "kodak") fmtHndl->subFormat = DCRAW_FORMAT_KODAK;
      if (make_string == "konika") fmtHndl->subFormat = DCRAW_FORMAT_MINOLTA;
      if (make_string == "nikon") fmtHndl->subFormat = DCRAW_FORMAT_NIKON;
      if (make_string == "olympus") fmtHndl->subFormat = DCRAW_FORMAT_OLYMPUS;
      if (make_string == "pentax") fmtHndl->subFormat = DCRAW_FORMAT_PENTAX;
      if (make_string == "fuji") fmtHndl->subFormat = DCRAW_FORMAT_FUJI;
      if (make_string == "rollei") fmtHndl->subFormat = DCRAW_FORMAT_ROLLEI;
      if (make_string == "sony") fmtHndl->subFormat = DCRAW_FORMAT_SONY;
      if (make_string == "sigma") fmtHndl->subFormat = DCRAW_FORMAT_SIGMA;
      */
    }
    else
    { 
      dimDCRawCloseImageProc( fmtHndl ); 
      return 1; 
    };

    //dimDCRawGetImageInfo( fmtHndl );
  }
  else return 1;

  return 0;
}

//----------------------------------------------------------------------------
// INFO for OPEN image
//----------------------------------------------------------------------------

DIM_UINT __stdcall dimDCRawGetNumPagesProc ( TDivFormatHandle *fmtHndl )
{
  if (fmtHndl == NULL) return 0;
  if (fmtHndl->internalParams == NULL) return 0;
  TDimDCRawParams *picPar = (TDimDCRawParams *) fmtHndl->internalParams;

  //return picPar->i.number_pages;
  return 1;
}


TDivImageInfo __stdcall dimDCRawGetImageInfoProc ( TDivFormatHandle *fmtHndl, DIM_UINT page_num )
{
  TDivImageInfo ii = initTDivImageInfo();

  if (fmtHndl == NULL) return ii;
  TDimDCRawParams *picPar = (TDimDCRawParams *) fmtHndl->internalParams;

  return picPar->i;
}

//----------------------------------------------------------------------------
// METADATA
//----------------------------------------------------------------------------

DIM_UINT __stdcall dimDCRawReadMetaDataProc (TDivFormatHandle *fmtHndl, DIM_UINT page, int group, int tag, int type)
{
  if (fmtHndl == NULL) return 1;
  //return read_biorad_metadata (fmtHndl, group, tag, type);
  //return 0;
  return 1;
}

char* __stdcall dimDCRawReadMetaDataAsTextProc ( TDivFormatHandle *fmtHndl )
{
  if (fmtHndl == NULL) return NULL;



  return (char *) read_dcraw_metatext( );
  return NULL;
}


//----------------------------------------------------------------------------
// READ/WRITE
//----------------------------------------------------------------------------

DIM_UINT __stdcall dimDCRawReadImageProc  ( TDivFormatHandle *fmtHndl, DIM_UINT page )
{
  if (fmtHndl == NULL) return 1;
  if (fmtHndl->stream == NULL) return 1;
  if (fmtHndl->internalParams == NULL) return 1;  
  TDimDCRawParams *picPar = (TDimDCRawParams *) fmtHndl->internalParams;

  fmtHndl->pageNumber = page;
  unsigned char *p;
  RGBQUAD lut[256];
  int quality = 0; // 0,1,2,3
  if (fmtHndl->quality == pos_to_quality[3]) quality = 3;
  else
  if (fmtHndl->quality > pos_to_quality[2])  quality = 2;
  else
  if (fmtHndl->quality > pos_to_quality[1])  quality = 1;

  if (dimCreateHBitmap( fmtHndl, picPar->i.width, picPar->i.height, 24, 0, lut, &p ) == 0)
    return read_cdraw_image( &p, getHBScanLine, fmtHndl->showProgressProc, quality, fmtHndl->testAbortProc ); 

  return 1;
}

DIM_UINT __stdcall dimDCRawWriteImageProc ( TDivFormatHandle *fmtHndl )
{
   
  return 1;
}


//****************************************************************************
// THUMBNAIL
//****************************************************************************

DIM_UINT __stdcall dimDCRawReadImageThumbProc (TDivFormatHandle *fmtHndl, DIM_UINT w, DIM_UINT h)
{
  RGBQUAD lut[256];
  unsigned char *p;
  unsigned int wi=0, he=0;

  try {
    if ( !get_info_thumb( &wi, &he ) ) return 1;

    if (dimCreateHBitmap( fmtHndl, wi, he, 24, 0, lut, &p ) == 0) {
      read_dcraw_thumb( &p, getHBScanLine );
      return 0;    
    }
  }
  catch (...) { return 1; }

  return 1;
}


//****************************************************************************
// Registry - saving/loading parameters
//****************************************************************************

const char regPath[] = "SOFTWARE\\DIMIN\\Viewer5\\Formats\\DCRAW\0";
const int buf_size = 4;

void __stdcall dimDCRawLoadRegProc ( TDivFormatHandle *fmtHndl ) {
  HKEY hKey;
  LONG lRet;
  DWORD bufLen = buf_size;
  int val;
  fmtHndl->quality = 0;
  lRet = RegOpenKeyEx( HKEY_CURRENT_USER, regPath, 0, KEY_QUERY_VALUE, &hKey );
  
  if (lRet==ERROR_SUCCESS) {
    if ( RegQueryValueEx(hKey, TEXT("Quality"), NULL, NULL, (LPBYTE) &val, &bufLen) == ERROR_SUCCESS )
      fmtHndl->quality = val;
  } // key open

  RegCloseKey( hKey );
}

void __stdcall dimDCRawSaveRegProc ( TDivFormatHandle *fmtHndl ) {
  HKEY hKey;
  LONG lRet;
  DWORD dwDisp;

  lRet = RegCreateKeyEx( HKEY_CURRENT_USER, regPath,
                         0, NULL, REG_OPTION_NON_VOLATILE,
                         KEY_WRITE, NULL, &hKey, &dwDisp );
  
  if (lRet==ERROR_SUCCESS) {
    int val = fmtHndl->quality;
    RegSetValueEx( hKey, TEXT("Quality"), NULL, REG_DWORD, (const BYTE *) &val, 4);  
  } // key open

  RegCloseKey( hKey );
}


//****************************************************************************
// GUI DEMANDED FUNTIONS
//****************************************************************************

WNDPROC dcrawWndProcOld; 
HWND dcrawComboBox;
HWND dcrawLabel1;
int  dcrawQuality = 0;

LRESULT CALLBACK dcrawWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  bool Result = false;
  
  int dcrawPos = SendMessage( dcrawComboBox, CB_GETCURSEL, 0, 0 );
  dcrawQuality = pos_to_quality[dcrawPos]; 

	if (!Result) 
    return CallWindowProc( dcrawWndProcOld, hwnd, message, wParam, lParam);
  else 
    return Result;
}

void __stdcall dimDCRawConfigInitProc ( TDivFormatHandle *fmtHndl, HWND hwndParent )
{
  HINSTANCE wHinst = (HINSTANCE) GetWindowLong( hwndParent, GWL_HINSTANCE );
 
  dcrawWndProcOld = (WNDPROC) GetWindowLong( hwndParent, GWL_WNDPROC );
	SetWindowLong( hwndParent, GWL_WNDPROC, (long) dcrawWndProc );
  
  dcrawComboBox = CreateWindow("combobox", "combobox1", 
     WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST | WS_TABSTOP | CBS_HASSTRINGS,
     10, 10, 250, 30,
     hwndParent, NULL, (HINSTANCE)GetWindowLong(hwndParent, GWL_HINSTANCE), NULL);

  SendMessage(dcrawComboBox, CB_ADDSTRING, 0, (long)"Fast (Linear)");
  SendMessage(dcrawComboBox, CB_ADDSTRING, 0, (long)"Normal (VNG)");
  SendMessage(dcrawComboBox, CB_ADDSTRING, 0, (long)"Quality (PPG)");
  SendMessage(dcrawComboBox, CB_ADDSTRING, 0, (long)"High Quality (AHD)");

  if (fmtHndl->quality == pos_to_quality[3])
    SendMessage(dcrawComboBox, CB_SETCURSEL, 3, 0 );
  else
  if (fmtHndl->quality > pos_to_quality[2])
    SendMessage(dcrawComboBox, CB_SETCURSEL, 2, 0 );
  else
  if (fmtHndl->quality > pos_to_quality[1])
    SendMessage(dcrawComboBox, CB_SETCURSEL, 1, 0 );
  else
    SendMessage(dcrawComboBox, CB_SETCURSEL, 0, 0 );
}

void __stdcall dimDCRawConfigQuitProc ( TDivFormatHandle *fmtHndl, HWND hwndParent )
{
  fmtHndl->quality = dcrawQuality;
  SetWindowLong( hwndParent, GWL_WNDPROC, (long) dcrawWndProcOld );
  DestroyWindow( dcrawComboBox );
  dimDCRawSaveRegProc ( fmtHndl );
}

