/*******************************************************************************

  Example Format Plugin Library (DLL) for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    12/05/2000 20:36:48 - First Creation
    03/25/2001 17:11:55 - Conversion to v2.1
    06/20/2004 01:10:00 - DIV plugin interface v5

*******************************************************************************/

#include <windows.h>

#include "div5_dcraw.h"
#include "dcraw/dim_dcraw_format.h"

TDIVPluginHeader PluginHeader = {
  sizeof( TDIVPluginHeader ),
  5, // plugin interface version - MUST BE 5
  DIV_PLUGIN_TYPE_FORMAT,
  4, // plugin version
  1, // one format, this is basically not used for this type
  "DIMIN DIV5 Digital Camera RAW Formats"
};

//------------------------------------------------------------------------------
// About procedure
//------------------------------------------------------------------------------
void __stdcall aboutPlugin( TDivFormatHandle *fmtHndl, HWND hwndParent ) 
{
  MessageBox(GetFocus(), "DIMIN Digital Camera's RAW (dcraw v8.93 rev 1.421)", "About Plug-in", MB_OK | MB_ICONEXCLAMATION);
}

//------------------------------------------------------------------------------
// Format definition
//------------------------------------------------------------------------------

TDivFormatItem divFormatItem[12] = {

  { //0
    "CANON-RAW",            // short name, no spaces
    "Canon Digital Camera RAW", // Long format name
    "cr2|crw",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 1
    "KODAK-RAW",            // short name, no spaces
    "Kodak Digital Camera RAW", // Long format name
    "bay|dc2|dcr|k25|kdc",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 2
    "MINOLTA-RAW",            // short name, no spaces
    "Konica/Minolta Digital Camera RAW", // Long format name
    "mrw",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 3
    "NIKON-RAW",            // short name, no spaces
    "Nikon Digital Camera RAW", // Long format name
    "nef",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 4
    "OLYMPUS-RAW",            // short name, no spaces
    "Olympus Digital Camera RAW", // Long format name
    "orf",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 5
    "PENTAX-RAW",            // short name, no spaces
    "Pentax Digital Camera RAW", // Long format name
    "pef",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 6
    "FUJI-RAW",            // short name, no spaces
    "Fuji Digital Camera RAW", // Long format name
    "raf",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 7
    "ROLLEI-RAW",            // short name, no spaces
    "Rollei Digital Camera RAW", // Long format name
    "rdc",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 8
    "SONY-RAW",            // short name, no spaces
    "Sony Digital Camera RAW", // Long format name
    "srf",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 9
    "SIGMA-RAW",            // short name, no spaces
    "Sigma Digital Camera RAW", // Long format name
    "x3f",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 10
    "DCRAW",            // short name, no spaces
    "Digital Cameras RAW", // Long format name
    "bmq|cs1|fff|mos",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
  ,{ // 11
    "DNG",            // short name, no spaces
    "Adobe Digital Negative", // Long format name
    "dng",   // pipe "|" separated supported extension list
    1, //canRead;      // 0 - NO, 1 - YES
    0, //canWrite;     // 0 - NO, 1 - YES
    0, //canReadMeta;  // 0 - NO, 1 - YES
    0, //canWriteMeta; // 0 - NO, 1 - YES
    0, //canAddMeta;   // 0 - NO, 1 - YES
    //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 0, 0, 0, 0, 0, 0 } 
  }
};

TDivFormatHeader formatHeader = {
  sizeof(TDivFormatHeader),      // must be == sizeof(TDivFormatHeader)
  "1.0.7",                  // plugin version, e.g. "1.0.0"
  "DCRAW Codec",       // plugin name
  "Digital Camera RAW codec for div5",    // plugin description
  
  10, //8, // 0 or more, specify number of bytes needed to identify the file // 8 - all, 56 - BioRad
  {1, 12, divFormatItem}, //TDivFormatList              supportedFormats;
  
  dimDCRawValidateFormatProc, //TDivValidateFormatProc      validateFormatProc; // return TRUE if can handle file
  // begin
  dimDCRawAquireFormatProc, //TDivAquireFormatProc        aquireFormatProc;
  // end
  dimDCRawReleaseFormatProc, //TDivReleaseFormatProc       releaseFormatProc;

  // params
  aboutPlugin, //TDivAboutFormatProc         aboutFormatProc;
  NULL, //TDivConfigFormatProc        configFormatProc;

  // image begin
  dimDCRawOpenImageProc, //TDivOpenImageProc           openImageProc; 
  dimDCRawCloseImageProc, //TDivCloseImageProc          closeImageProc; 

  // info
  dimDCRawGetNumPagesProc, //TDivGetNumPagesProc         getNumPagesProc;
  dimDCRawGetImageInfoProc, //TDivGetImageInfoProc        getImageInfoProc;

  // read/write
  dimDCRawReadImageProc, //TDivReadImageProc           readImageProc; 
  dimDCRawWriteImageProc, //TDivWriteImageProc          writeImageProc;
  NULL, //TDivReadImageTileProc       readImageTileProc;
  NULL, //TDivWriteImageTileProc      writeImageTileProc;
  NULL, //TDivReadImageLineProc       readImageLineProc;
  NULL, //TDivWriteImageLineProc      writeImageLineProc;
  dimDCRawReadImageThumbProc, //TDivReadImageThumbProc      readImageThumbProc;
  NULL, //TDivWriteImageThumbProc     writeImageThumbProc;
  NULL, //TDivReadImagePreviewProc    readImagePreviewProc;
  
  // meta data
  NULL, //TDivReadMetaDataProc        readMetaDataProc;
  NULL, //TDivAddMetaDataProc         addMetaDataProc;
  dimDCRawReadMetaDataAsTextProc, //TDivReadMetaDataAsTextProc  readMetaDataAsTextProc;

  dimDCRawConfigInitProc, //dimDCRawConfigInitProc, //TDivConfigFormatProc
  dimDCRawConfigQuitProc, //dimDCRawConfigQuitProc, //TDivConfigFormatProc

  // xtd
  NULL, // xtdProc

  NULL, // p1
  NULL, // p2
  NULL, // p3

};

//---------------------------------------------------------------------------

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
  switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


TDIVPluginHeader* __stdcall divGetPluginHeader()
{
  return &PluginHeader;
}

TDivFormatHeader* __stdcall divGetFormatHeader()
{
   return &formatHeader;
}



//---------------------------------------------------------------------------
